@@ -1,3 +1,20 @@
+# Copyright 2009-2013 Justin Riley
+#
+# This file is part of StarCluster.
+#
+# StarCluster is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option) any
+# later version.
+#
+# StarCluster is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with StarCluster. If not, see <http://www.gnu.org/licenses/>.
+
 import glob
 
 from starcluster import exception
@@ -31,7 +48,7 @@ class CmdGet(ClusterCompleter):
         parser.add_option("-u", "--user", dest="user", default=None,
                           help="Transfer files as USER ")
         parser.add_option("-n", "--node", dest="node", default="master",
-                          help="Transfer files to NODE (defaults to master)")
+                          help="Transfer files from NODE (defaults to master)")
 
     def execute(self, args):
         if len(args) < 3:
@@ -40,7 +57,7 @@ class CmdGet(ClusterCompleter):
         ctag = args[0]
         lpath = args[-1]
         rpaths = args[1:-1]
-        cl = self.cm.get_cluster(ctag)
+        cl = self.cm.get_cluster(ctag, load_receipt=False)
         node = cl.get_node_by_alias(self.opts.node)
         if self.opts.user:
             node.ssh.switch_user(self.opts.user)
