@@ -1,3 +1,20 @@
+# Copyright 2009-2013 Justin Riley
+#
+# This file is part of StarCluster.
+#
+# StarCluster is free software: you can redistribute it and/or modify it under
+# the terms of the GNU Lesser General Public License as published by the Free
+# Software Foundation, either version 3 of the License, or (at your option) any
+# later version.
+#
+# StarCluster is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
+# details.
+#
+# You should have received a copy of the GNU Lesser General Public License
+# along with StarCluster. If not, see <http://www.gnu.org/licenses/>.
+
 from starcluster import exception
 from starcluster.balancers import sge
 
@@ -49,31 +66,39 @@ class CmdLoadBalance(ClusterCompleter):
                           "(default: %s)" % sge.DEFAULT_STATS_DIR %
                           "<cluster_tag>")
         parser.add_option("-i", "--interval", dest="interval",
-                          action="store", type="int", default=None,
+                          action="callback", type="int", default=None,
+                          callback=self._positive_int,
                           help="Load balancer polling interval in seconds "
                           "(max: 300s)")
         parser.add_option("-m", "--max_nodes", dest="max_nodes",
-                          action="store", type="int", default=None,
+                          action="callback", type="int", default=None,
+                          callback=self._positive_int,
                           help="Maximum # of nodes in cluster")
         parser.add_option("-w", "--job_wait_time", dest="wait_time",
-                          action="store", type="int", default=None,
+                          action="callback", type="int", default=None,
+                          callback=self._positive_int,
                           help=("Maximum wait time for a job before "
                                 "adding nodes, seconds"))
         parser.add_option("-a", "--add_nodes_per_iter", dest="add_pi",
-                          action="store", type="int", default=None,
+                          action="callback", type="int", default=None,
+                          callback=self._positive_int,
                           help="Number of nodes to add per iteration")
         parser.add_option("-k", "--kill_after", dest="kill_after",
-                          action="store", type="int", default=None,
+                          action="callback", type="int", default=None,
+                          callback=self._positive_int,
                           help="Minutes after which a node can be killed")
         parser.add_option("-s", "--stabilization_time", dest="stab",
-                          action="store", type="int", default=None,
+                          action="callback", type="int", default=None,
+                          callback=self._positive_int,
                           help="Seconds to wait before cluster "
                           "stabilizes (min: 300s)")
         parser.add_option("-l", "--lookback_window", dest="lookback_win",
-                          action="store", type="int", default=None,
+                          action="callback", type="int", default=None,
+                          callback=self._positive_int,
                           help="Minutes to look back for past job history")
         parser.add_option("-n", "--min_nodes", dest="min_nodes",
-                          action="store", type="int", default=None,
+                          action="callback", type="int", default=None,
+                          callback=self._positive_int,
                           help="Minimum number of nodes in cluster")
         parser.add_option("-K", "--kill-cluster", dest="kill_cluster",
                           action="store_true", default=False,
