@@ -105,7 +105,7 @@ CLUSTER_SHELL = bash
 # The base i386 StarCluster AMI is %(x86_ami)s
 # The base x86_64 StarCluster AMI is %(x86_64_ami)s
 # The base HVM StarCluster AMI is %(hvm_ami)s
-NODE_IMAGE_ID = %(x86_ami)s
+NODE_IMAGE_ID = %(x86_64_ami)s
 # instance type for all cluster nodes
 # (options: %(instance_types)s)
 NODE_INSTANCE_TYPE = m1.small
@@ -117,7 +117,7 @@ NODE_INSTANCE_TYPE = m1.small
 #MASTER_INSTANCE_TYPE = m1.small
 # Uncomment to specify a separate AMI to use for the master node. (OPTIONAL)
 # (defaults to NODE_IMAGE_ID if not specified)
-#MASTER_IMAGE_ID = %(x86_ami)s (OPTIONAL)
+#MASTER_IMAGE_ID = %(x86_64_ami)s (OPTIONAL)
 # availability zone to launch the cluster in (OPTIONAL)
 # (automatically determined based on volumes (if any) or
 # selected by Amazon if not specified)
@@ -205,32 +205,32 @@ NODE_INSTANCE_TYPE = m1.small
 ## Configuring Security Group Permissions ##
 ############################################
 # Sections starting with "permission" define security group rules to
-# automatically apply to newly created clusters. PROTOCOL in the following
+# automatically apply to newly created clusters. IP_PROTOCOL in the following
 # examples can be can be: tcp, udp, or icmp. CIDR_IP defaults to 0.0.0.0/0 or
 # "open to the # world"
 
 # open port 80 on the cluster to the world
 # [permission http]
-# PROTOCOL = tcp
+# IP_PROTOCOL = tcp
 # FROM_PORT = 80
 # TO_PORT = 80
 
 # open https on the cluster to the world
 # [permission https]
-# PROTOCOL = tcp
+# IP_PROTOCOL = tcp
 # FROM_PORT = 443
 # TO_PORT = 443
 
 # open port 80 on the cluster to an ip range using CIDR_IP
 # [permission http]
-# PROTOCOL = tcp
+# IP_PROTOCOL = tcp
 # FROM_PORT = 80
 # TO_PORT = 80
 # CIDR_IP = 18.0.0.0/8
 
 # restrict ssh access to a single ip address (<your_ip>)
 # [permission ssh]
-# PROTOCOL = tcp
+# IP_PROTOCOL = tcp
 # FROM_PORT = 22
 # TO_PORT = 22
 # CIDR_IP = <your_ip>/32
